from p4a.plonevideoembed import interfaces
from p4a.plonevideoembed import content
from p4a.common import site
from p4a.z2utils import utils

from Products.CMFCore import utils as cmfutils

import logging
logger = logging.getLogger('p4a.plonevideoembed.sitesetup')

try:
    import five.localsitemanager
    HAS_FLSM = True
    logger.info('Using five.localsitemanager')
except ImportError, err:
    HAS_FLSM = False

def setup_portal(portal):
    site.ensure_site(portal)
    setup_site(portal)
    setup_smart_folder_indexes(portal)

    qi = cmfutils.getToolByName(portal, 'portal_quickinstaller')
    qi.installProducts(['CMFonFive'])

INDEX_MAPPING = {'object_provides':
                    {'name': 'Interfaces provided',
                     'description': 'All interfaces provided by the given '
                                    'object.',
                     'enabled': True,
                     'criteria': ('ATSimpleStringCriterion',
                                  'ATListCriterion',)},
                 }

def setup_smart_folder_indexes(portal):
    """Adds the default indexes to be available from smartfolders"""

    atct_config = cmfutils.getToolByName(portal, 'portal_atct')

    for index, index_info in INDEX_MAPPING.items():
        atct_config.updateIndex(index, friendlyName=index_info['name'],
                                description=index_info['description'],
                                enabled=index_info['enabled'],
                                criteria=index_info['criteria'])
        atct_config.updateMetadata(index, friendlyName=index_info['name'],
                                   description=index_info['description'],
                                   enabled=True)

def setup_site(site):
    """Install all necessary components and configuration into the
    given site.

      >>> from p4a.plonevideoembed import interfaces
      >>> from p4a.common.testing import MockSite

      >>> site = MockSite()
      >>> site.queryUtility(interfaces.IVideoLinkSupport) is None
      True

      >>> setup_site(site)
      >>> site.getUtility(interfaces.IVideoLinkSupport)
      <VideoLinkSupport ...>

    """

    sm = site.getSiteManager()
    if not sm.queryUtility(interfaces.IVideoLinkSupport):
        if HAS_FLSM:
            sm.registerUtility(content.VideoLinkSupport('video_support'),
                               interfaces.IVideoLinkSupport)
        else:
            sm.registerUtility(interfaces.IVideoLinkSupport,
                               content.VideoLinkSupport('video_support'))

def _cleanup_utilities(site):
    raise NotImplementedError('Current ISiteManager support does not '
                              'include ability to clean up')

def unsetup_portal(portal):
    count = utils.remove_marker_ifaces(portal, interfaces.IVideoLinkEnhanced)
    logger.warn('Removed IVideoLinkEnhanced interface from %i objects for '
              'cleanup' % count)



from zope import component
from zope import interface
from p4a.plonevideoembed import interfaces
from p4a.common import feature

_marker = object()

class MediaActivator(object):
    """An adapter for seeing the activation status or toggling activation.
    """

    interface.implements(interfaces.IMediaActivator)
    component.adapts(interface.Interface)

    def __init__(self, context):
        self.context = context

    _video_link_activated = feature.FeatureProperty(
        interfaces.IAnyVideoLinkCapable,
        interfaces.IVideoLinkEnhanced,
        'context')

    def media_activated(self, v=_marker):
        if v is _marker:
            if interfaces.IAnyVideoLinkCapable.providedBy(self.context):
                return self._video_link_activated
            return False

        if interfaces.IAnyVideoLinkCapable.providedBy(self.context):
            self._video_link_activated = v

    media_activated = property(media_activated, media_activated)



import doctest
import unittest

from zope.testing import doctestunit
from zope.component import testing

def test_suite():
    return unittest.TestSuite([
        doctestunit.DocTestSuite('p4a.plonevideoembed.content'),

        doctestunit.DocFileSuite('plonevideoembed.txt',
                                 package='p4a.plonevideoembed',
                                 setUp=testing.setUp,
                                 tearDown=testing.tearDown),
        doctestunit.DocFileSuite('sitesetup.txt',
                                 package='p4a.plonevideoembed',
                                 optionflags=doctest.ELLIPSIS,
                                 setUp=testing.setUp,
                                 tearDown=testing.tearDown),
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from zope import component
from zope import interface
from p4a.videoembed import interfaces as baseifaces
from p4a.plonevideoembed import interfaces
import Acquisition

def attempt_videolink_activation(obj):
    """Determine if the link in question contains information that videoembed
    knows about, and if so, activate the link as media_activated=True.
    """

    activation = interfaces.IMediaActivator(obj)

    if activation.media_activated:
        return False

    find_type = component.getUtility(baseifaces.IURLType)
    provider = baseifaces.ILinkProvider(Acquisition.aq_base(obj))
    url = provider.getLink()

    found = find_type(url)
    if found:
        activation.media_activated = True

    return activation.media_activated

def autoactivation_handler(obj, event):
    attempt_videolink_activation(obj)



from zope import interface
from zope import schema

class ILinkHolder(interface.Interface):
    """A zope schema based interface for getting and setting a url.
    """

    url = schema.TextLine(title=u'Video URL',
                          required=False)

class IVideoLinkSupport(interface.Interface):
    """Provides certain information about video support.
    """

    support_enabled = schema.Bool(title=u'Video Support Enabled?',
                                  required=True,
                                  readonly=True)

class IAnyVideoLinkCapable(interface.Interface):
    """Marker interface declaring that a particular class/object can have
    link information pointing to video.
    """

class IVideoLinkEnhanced(interface.Interface):
    """Marker interface declaring that a particular class/object is
    identified as being a link to video.
    """

class IMediaActivator(interface.Interface):
    """For seeing the activation status or toggling activation."""

    media_activated = schema.Bool(title=u'Media Activated',
                                  required=True,
                                  readonly=False)



from zope import interface
from zope import component
from Products.ATContentTypes.content import link
from p4a.videoembed import interfaces as embedifaces
from p4a.plonevideoembed import interfaces

class ATLinkLinkHolder(object):
    interface.implements(interfaces.ILinkHolder)
    component.adapts(link.ATLink)

    def __init__(self, context):
        self.context = context

    def _set_url(self, url):
        self.context.setRemoteUrl(url)
    def _get_url(self):
        # we're bypassing the accessor as it's currently
        # converting & in the url to %26 which is bad
        url = self.context.remoteUrl

        if isinstance(url, unicode):
            # registered embed adapters fail when looking up on unicode
            # because they've been registered against str
            url = url.encode('utf-8')

        return url
    url = property(_get_url, _set_url)

class ATLinkLinkProvider(object):
    interface.implements(embedifaces.ILinkProvider)
    component.adapts(link.ATLink)

    def __init__(self, context):
        self.context = context

    def getLink(self):
        holder = component.getAdapter(self.context, interfaces.ILinkHolder)
        return holder.url



from zope import interface
from p4a.plonevideoembed import interfaces
from OFS.SimpleItem import SimpleItem

class VideoLinkSupport(SimpleItem):
    """Simple persistent utility that indicates video link functionality
    is available.

      >>> VideoLinkSupport('foo').support_enabled
      True

    """

    interface.implements(interfaces.IVideoLinkSupport)

    @property
    def support_enabled(self):
        return True



import Acquisition
from zope import component
from zope.formlib import form
from zope import interface
from zope import schema
from p4a.plonevideoembed import interfaces
from p4a.plonevideoembed import media
from p4a.video import interfaces as videoifaces
from p4a.videoembed import interfaces as embedifaces
from p4a.common import feature
import p4a.video.browser.video
from p4a.video.browser.video import has_contenttagging_support
from p4a.video.browser.video import has_contentrating_support

class IContextualVideoLinkSupport(interface.Interface):
    can_activate_video_link = schema.Bool(title=u'Can Activate Video Link',
                                          readonly=True)
    can_deactivate_video_link = schema.Bool(title=u'Can Deactivate Video Link',
                                            readonly=True)

class Support(object):
    """A view that returns certain information regarding p4avideoembed status.
    """

    interface.implements(IContextualVideoLinkSupport)
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        
    @property
    def support_enabled(self):
        """Check to make sure an IVideoLinkSupport utility is available and
        if so, query it to determine if support is enabled.
        """
        
        support = component.queryUtility(interfaces.IVideoLinkSupport)
        if support is None:
            return False

        return support.support_enabled

    @property
    def _basic_can(self):
        if not self.support_enabled:
            return False

        if not interfaces.IAnyVideoLinkCapable.providedBy(self.context):
            return False

        return True

    @property
    def can_activate_video_link(self):
        if not self._basic_can:
            return False
        
        mediaconfig = component.getMultiAdapter((self.context, self.request),
                                                name='video-link-config.html')
        return not mediaconfig.media_activated

    @property
    def can_deactivate_video_link(self):
        if not self._basic_can:
            return False

        mediaconfig = component.getMultiAdapter((self.context, self.request),
                                                name='video-link-config.html')
        return mediaconfig.media_activated

_marker = object()

class ToggleEnhancementsView(media.MediaActivator):
    """
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        was_activated = self.media_activated
        self.media_activated = not was_activated
        response = self.request.response

        if was_activated:
            activated = 'Media+deactivated'
        else:
            activated = 'Media+activated'

        baseurl = self.context.absolute_url()
        response.redirect(baseurl+'/view?portal_status_message='+activated)

class VideoLinkView(object):
    """The view for a video link object.
    """

    def has_contentrating_support(self):
        return has_contentrating_support(Acquisition.aq_inner(self.context))

    def has_contenttagging_support(self):
        return has_contenttagging_support(Acquisition.aq_inner(self.context))

    def video_author(self):
        video = videoifaces.IVideo(Acquisition.aq_inner(self.context))
        return getattr(video, 'video_author', '')

    def video_width(self):
        video = videoifaces.IVideo(Acquisition.aq_inner(self.context))
        return getattr(video, 'width', 0)

    def video_height(self):
        video = videoifaces.IVideo(Acquisition.aq_inner(self.context))
        return getattr(video, 'height', 0)

    def url(self):
        holder = component.getAdapter(self.context, interfaces.ILinkHolder)
        return holder.url

class VideoLinkEditForm(p4a.video.browser.video.VideoEditForm):
    """Edit video link"""

    form_fields = form.Fields(
        videoifaces.IVideo['title'],
        interfaces.ILinkHolder['url'],
        videoifaces.IVideo['description'],
        videoifaces.IVideo['video_image'],
        videoifaces.IVideo['video_author'],
        videoifaces.IVideo['width'],
        )

    priority_fields = ['title', 'url']



from setuptools import setup, find_packages
import sys, os

version = '1.0'

setup(name='p4a.plonevideoembed',
      version=version,
      description="Plone4Artists video embedding add-on for Plone",
      long_description="""p4a.plonevideo is a video embedding add-on for the Plone
CMS.""",
      classifiers=[
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Plone Foundation',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://svn.plone.org/svn/plone/plone.example',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['p4a'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
